#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "log.h"
#include "file_utils.h"
#include "linkguard_sdk.h"

#define PRODUCT_SECRET      "0ESGfQks0SIbBhkR"
int g_running_flag = 1;

void sig_handler(int signo) {
    log_info("catch the signal SIGINT %d\r\n", signo);
    g_running_flag = 0;
}

static int del_all_user() {
    log_info("delete all user done\r\n");
    return 0;
}

static int add_user(const struct iotx_user_info_t *user_info) {
    log_info("add user_info to local database: %s\r\n", user_info->id);
    for (int i = 0; i < user_info->pic_feature_size; ++i) {
        log_info("user_info feature: %s\r\n", user_info->pic_feature[i]);
    }
    return 0;
}

static int del_user(const char *person_id) {
    log_info("del person from local database: %s\r\n", person_id);
    return 0;
}

extern int linksdk_init_adapter(struct iotx_lg_config_t *cfg);

extern int linksdk_destroy_adapter();

int main(int argc, char **argv) {
    char *pk = NULL;
    char *dn = NULL;
    char *ds = NULL;
    char *data_path = NULL;
    char *log_path = NULL;
    if (argc < 5) {
        log_warn("please run like this: ./linkguard_demo ProductKey DeviceName DeviceSecret data_path log_path\r\n");
        return 0;
    }
    pk = argv[1];
    dn = argv[2];
    ds = argv[3];
    data_path = argv[4];
    log_path = argv[5];

    log_info("args: %s %s %s %s %s\r\n", pk, dn, ds, data_path, log_path);
    if (signal(SIGINT, sig_handler) == SIG_ERR) {
        perror("[LG-SDK] signal error\r\n");
        exit(EXIT_FAILURE);
    }
    register_add_user(add_user);
    register_delete_user(del_user);
    register_delete_all(del_all_user);

    char *mqtt_url = "iot-as-mqtt.cn-shanghai.aliyuncs.com";
    char *http2_url = "iot-as-http2.cn-shanghai.aliyuncs.com";
    struct iotx_log_cfg_t log_cfg = {
            .lvl = LF_LOG_INFO,
            .dst = LF_FILE,
            .log_path = log_path,
            .max_size_log_mb = 10
    };
    struct iotx_lg_config_t config = {
            .public_instance = 1,
            .mqtt_url = mqtt_url,
            .http2_url = http2_url,
            .product_key = pk,
            .device_name = dn,
            .product_secret = PRODUCT_SECRET,
            .device_secret = ds,
            .data_path = data_path,
            .cfg_log = log_cfg,
            .update_mode = 1
    };

    _retry:
#ifdef ADAPTER_MODE
    log_info("adapter mode init\r\n");
    if (1 != linksdk_init_adapter(&config)) {
        log_error("[LG-SDK] init failed, retry 3s later\r\n");
        sleep(3);
        goto _retry;
    }

    if (1 != iotx_lg_init(&config)) {
        log_info("init failed, retry 3s later\r\n");
        sleep(3);
        iotx_lg_deinit();
        linksdk_destroy_adapter();
        goto _retry;
    }
#else
    log_info("singleton mode init\r\n");
    if (1 != iotx_lg_init(&config)) {
        log_info("iotx_lg_init failed, retry 3s later\r\n");
        sleep(3);
        iotx_lg_deinit();
        goto _retry;
    }
#endif
    log_info("link guard sdk init done\r\n");

    // 读取比对成功图片
    char success_path[128] = {0};
    strcat(success_path, data_path);
    strcat(success_path, "/match_success.png");
    unsigned int success_content_size = 0;
    char *success_content = read_file_content(success_path, &success_content_size, "r");
    if (success_content == NULL) {
        log_warn("file %s is empty\r\n", success_path);
        return 0;
    }

    // 读取比对失败图片
    char failed_path[128] = {0};
    strcat(failed_path, data_path);
    strcat(failed_path, "/match_failed.png");
    unsigned int failed_content_size = 0;
    char *failed_content = read_file_content(failed_path, &failed_content_size, "r");
    if (failed_content == NULL) {
        log_warn("file %s is empty\r\n", failed_path);
        return 0;
    }

    // 事件上报case
    struct iotx_matche_result_t realtime_res = {
            .panorama_pic = failed_content,
            .panorama_pic_name = "realtime_match_failed.png",
            .panorama_pic_size = failed_content_size,
            .device_id = "device_id",
            .match_status = MATCH_STATUS_FAILED,
            .is_realtime = 1,
    };
    struct iotx_matche_result_t match_realtime_res = {
            .panorama_pic = success_content,
            .panorama_pic_name = "realtime_match_success.png",
            .panorama_pic_size = success_content_size,
            .device_id = "device_id",
            .match_status = MATCH_STATUS_SUCCESS,
            .is_realtime = 1,
            .user_id = "lhbpbph129q3prod"
    };
    struct iotx_matche_result_t none_realtime_res = {
            .panorama_pic = failed_content,
            .panorama_pic_name = "none_realtime_match_failed.png",
            .panorama_pic_size = failed_content_size,
            .device_id = "device_id",
            .is_realtime = 0,
            .time_msec = 1626871517000,
            .match_status = MATCH_STATUS_FAILED
    };
    struct iotx_matche_result_t match_none_realtime_res = {
            .panorama_pic = success_content,
            .panorama_pic_name = "none_realtime_match_success.png",
            .panorama_pic_size = success_content_size,
            .device_id = "device_id",
            .match_status = MATCH_STATUS_SUCCESS,
            .is_realtime = 0,
            .time_msec = 1626871517000,
            .user_id = "lhbpbph129q3prod"
    };

    // 保持设备连接状态
    while (1 == g_running_flag) {
        sleep(30);
        int ret = upload_match_result(&realtime_res);
        log_info("realtime_ret %d\r\n", ret);
        ret = upload_match_result(&match_realtime_res);
        log_info("match_realtime_ret %d\r\n", ret);
        ret = upload_match_result(&none_realtime_res);
        log_info("none_realtime_ret %d\r\n", ret);
        ret = upload_match_result(&match_none_realtime_res);
        log_info("match_none_realtime %d\r\n", ret);
        none_realtime_res.time_msec += 30000;
        match_none_realtime_res.time_msec += 30000;
    }

    // 单次上报事件
    int ret = upload_match_result(&realtime_res);
    log_info("realtime_ret %d\r\n", ret);
    ret = upload_match_result(&match_realtime_res);
    log_info("match_realtime_ret %d\r\n", ret);
    ret = upload_match_result(&none_realtime_res);
    log_info("none_realtime_ret %d\r\n", ret);
    ret = upload_match_result(&match_none_realtime_res);
    log_info("match_none_realtime %d\r\n", ret);

#ifdef ADAPTER_MODE
    log_info("adapter mode destroy\r\n");
    linksdk_destroy_adapter();
#else
    log_info("singleton mode destroy\r\n");
#endif

    free(success_content);
    free(failed_content);
    iotx_lg_deinit();
    return 0;
}