#ifndef __SEMAPHORE_PORT_H__
#define __SEMAPHORE_PORT_H__

#include <errno.h>

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef __APPLE__

#include <dispatch/dispatch.h>

#else
#include <semaphore.h>
#endif

struct lg_sem_t {
#ifdef __APPLE__
    dispatch_semaphore_t sem;
#else
    sem_t                   sem;
#endif
};

static inline void lg_sem_init(struct lg_sem_t *s, uint32_t value) {
#ifdef __APPLE__
    dispatch_semaphore_t *sem = &s->sem;
    *sem = dispatch_semaphore_create(value);
#else
    sem_init(&s->sem, 0, value);
#endif
}

static inline void lg_sem_wait(struct lg_sem_t *s) {
#ifdef __APPLE__
    dispatch_semaphore_wait(s->sem, DISPATCH_TIME_FOREVER);
#else
    int r;
    do {
            r = sem_wait(&s->sem);
    } while (r == -1 && errno == EINTR);
#endif
}

static inline void lg_sem_post(struct lg_sem_t *s) {
#ifdef __APPLE__
    dispatch_semaphore_signal(s->sem);
#else
    sem_post(&s->sem);
#endif
}

static inline void lg_sem_destroy(struct lg_sem_t *s) {
#ifdef __APPLE__
    dispatch_release(s->sem);
#else
    sem_destroy(&s->sem);
#endif
}

#if defined(__cplusplus)
}
#endif

#endif //__SEMAPHORE_PORT_H__